Exception: Porolog::Goal::Error
- Inherits:
-
PorologError
- Object
- RuntimeError
- PorologError
- Porolog::Goal::Error
- Defined in:
- lib/porolog/goal.rb
Overview
Error class for rescuing or detecting any Goal error.
Error class for rescuing or detecting any Goal error.