Exception: Porolog::NoGoalError
- Inherits:
-
PorologError
- Object
- RuntimeError
- PorologError
- Porolog::NoGoalError
- Defined in:
- lib/porolog/error.rb
Overview
Error indicating that a Goal is needed but could not be found.
Error indicating that a Goal is needed but could not be found.